package blx::xsdsql::xsd_parser::node::whiteSpace;
use base qw(blx::xsdsql::xsd_parser::type_restriction);
use strict;
use warnings;
use integer;



1;

__END__

=head1  NAME

blx::xsdsql::xsd_parser::node::whiteSpace - internal class for parsing schema 

=cut



