package blx::xsdsql::xsd_parser::node::pattern;
use base qw(blx::xsdsql::xsd_parser::node);
use strict;
use warnings;
use integer;

1;

__END__


=head1  NAME

blx::xsdsql::xsd_parser::node::pattern - internal class for parsing schema 

=cut
