package blx::xsdsql::xsd_parser::node::documentation;
use base qw(blx::xsdsql::xsd_parser::node);
use strict;
use warnings;
use integer;



1;

__END__


=head1  NAME

blx::xsdsql::xsd_parser::node::documentation - internal class for parsing schema 

=cut
