package blx::xsdsql::xsd_parser::node::simpleContent;
use base qw(blx::xsdsql::xsd_parser::type_restriction);
use strict;
use warnings;
use integer;
use Carp;



1;



__END__

=head1  NAME

blx::xsdsql::xsd_parser::type::simple_content - internal class for parsing schema 

=cut


