package blx::xsdsql::xsd_parser::node::minLength;
use base qw(blx::xsdsql::xsd_parser::type_restriction);
use strict;
use warnings;
use integer;
use Carp;


1;


__END__


=head1  NAME

blx::xsdsql::xsd_parser::node::minLength - internal class for parsing schema 

=cut
